Attribute VB_Name = "InputEvents"
Option Explicit

' I learned how to write this code from:
' http://www.vbapi.com/vbapi/ref/s/sendinput.html

' Win95 declarations
' Declare Sub keybd_event Lib "user32.dll" (ByVal bVk As Byte, ByVal bScan As Byte, ByVal dwFlags As Long, ByVal dwExtraInfo As Long)
' Declare Function SendInput Lib "user32.dll" (ByVal nInputs As Long, pInputs As INPUT_TYPE, ByVal cbSize As Long) As Long

' Win98/NT/etc declarations
Public Declare Function SendInput Lib "user32.dll" (ByVal nInputs As Long, pInputs As INPUT_TYPE, ByVal cbSize As Long) As Long
Public Declare Sub CopyMemory Lib "kernel32.dll" Alias "RtlMoveMemory" (Destination As Any, Source As Any, ByVal length As Long)

Public Type MOUSEINPUT
      dx As Long
      dy As Long
      mouseData As Long
      dwFlags As Long
      time As Long
      dwExtraInfo As Long
End Type

Public Const MOUSEEVENTF_ABSOLUTE = &H8000
Public Const MOUSEEVENTF_LEFTDOWN = &H2
Public Const MOUSEEVENTF_LEFTUP = &H4
Public Const MOUSEEVENTF_MIDDLEDOWN = &H20
Public Const MOUSEEVENTF_MIDDLEUP = &H40
Public Const MOUSEEVENTF_MOVE = &H1
Public Const MOUSEEVENTF_RIGHTDOWN = &H8
Public Const MOUSEEVENTF_RIGHTUP = &H10
Public Const MOUSEEVENTF_WHEEL = &H80
Public Const MOUSEEVENTF_XDOWN = &H100
Public Const MOUSEEVENTF_XUP = &H200
Public Const WHEEL_DELTA = 120
Public Const XBUTTON1 = &H1
Public Const XBUTTON2 = &H2


Public Type KEYBDINPUT
      wVk As Integer
      wScan As Integer
      dwFlags As Long
      time As Long
      dwExtraInfo As Long
End Type

Public Const KEYEVENTF_KEYUP = &H2

Public Type INPUT_TYPE
      dwType As Long
      xi(0 To 23) As Byte
End Type

Public Const INPUT_KEYBOARD = 1
Public Const INPUT_MOUSE = 0


Public Sub PressKey(KeyCode As Integer)

    'special handler for mouse buttons
    Select Case KeyCode
    Case vbKeyRButton
        PressMouseButton MOUSEEVENTF_RIGHTDOWN
    Case vbKeyLButton
        PressMouseButton MOUSEEVENTF_LEFTDOWN
    Case vbKeyMButton
        PressMouseButton MOUSEEVENTF_MIDDLEDOWN

    Case Else  'now do regular keyboard things
    
        Dim InputEvents(0 To 0) As INPUT_TYPE  ' holds information about each event
        Dim keyevent As KEYBDINPUT             ' temporarily hold keyboard input info
        
        ' Press the key
        With keyevent
            .wVk = KeyCode              ' the key to release
            .wScan = 0                  ' not needed
            .dwFlags = 0                ' press the key down
            .time = 0                   ' use the default
            .dwExtraInfo = 0            ' not needed
        End With
        InputEvents(0).dwType = INPUT_KEYBOARD
        CopyMemory InputEvents(0).xi(0), keyevent, Len(keyevent)
        
        ' Now that all the information for the two input events has been placed
        ' into the array, finally send it into the input stream.
        Dim X As Long
        X = SendInput(1, InputEvents(0), Len(InputEvents(0)))

    End Select
    
End Sub

Public Sub ReleaseKey(KeyCode As Integer)

    'special handler for mouse buttons
    Select Case KeyCode
    Case vbKeyRButton
        ReleaseMouseButton MOUSEEVENTF_RIGHTUP
    Case vbKeyLButton
        ReleaseMouseButton MOUSEEVENTF_LEFTUP
    Case vbKeyMButton
        ReleaseMouseButton MOUSEEVENTF_MIDDLEUP

    Case Else 'now do regular keyboard things
        
        Dim InputEvents(0 To 0) As INPUT_TYPE  ' holds information about each event
        Dim keyevent As KEYBDINPUT             ' temporarily hold keyboard input info
        
        ' Do the same as above, but for releasing the key.
        With keyevent
            .wVk = KeyCode              ' the key to release
            .wScan = 0                  ' not needed
            .dwFlags = KEYEVENTF_KEYUP  ' release the key
            .time = 0                   ' use the default
            .dwExtraInfo = 0            ' not needed
        End With
        InputEvents(0).dwType = INPUT_KEYBOARD
        CopyMemory InputEvents(0).xi(0), keyevent, Len(keyevent)
        
        ' Now that all the information for the two input events has been placed
        ' into the array, finally send it into the input stream.
        Dim X As Long
        X = SendInput(1, InputEvents(0), Len(InputEvents(0)))

    End Select
    
End Sub


Public Sub PressMouseButton(KeyCode As Integer)
    
    Dim InputEvents(0 To 0) As INPUT_TYPE  ' holds information about each event
    Dim mouseEvent As MOUSEINPUT           ' temporarily hold input info

    ' Load the information needed to imitate pressing the right mouse button.
    With mouseEvent
        .dx = 0            ' no horizontal movement
        .dy = 0            ' no vertical movement
        .mouseData = 0     ' not needed
        .dwFlags = KeyCode ' button down
        .time = 0          ' use the default
        .dwExtraInfo = 0   ' not needed
    End With
    
    ' Copy the structure into the input array's buffer.
    InputEvents(0).dwType = INPUT_MOUSE
    CopyMemory InputEvents(0).xi(0), mouseEvent, Len(mouseEvent)

    ' Now that all the information for the two input events has been placed
    ' into the array, finally send it into the input stream.
    Dim X As Long
    X = SendInput(1, InputEvents(0), Len(InputEvents(0)))

End Sub


Public Sub ReleaseMouseButton(KeyCode As Integer)
    
    Dim InputEvents(0 To 0) As INPUT_TYPE  ' holds information about each event
    Dim mouseEvent As MOUSEINPUT           ' temporarily hold input info

    ' Load the information needed to imitate pressing the right mouse button.
    With mouseEvent
        .dx = 0            ' no horizontal movement
        .dy = 0            ' no vertical movement
        .mouseData = 0     ' not needed
        .dwFlags = KeyCode ' button up?
        .time = 0          ' use the default
        .dwExtraInfo = 0   ' not needed
    End With
    
    ' Copy the structure into the input array's buffer.
    InputEvents(0).dwType = INPUT_MOUSE
    CopyMemory InputEvents(0).xi(0), mouseEvent, Len(mouseEvent)

    ' Now that all the information for the two input events has been placed
    ' into the array, finally send it into the input stream.
    Dim X As Long
    X = SendInput(1, InputEvents(0), Len(InputEvents(0)))

End Sub

